@@ -1,10 +1,15 @@
 # $Id: gre.py 75 2010-08-03 14:42:19Z jon.oberheide $
 # -*- coding: utf-8 -*-
 """Generic Routing Encapsulation."""
+from __future__ import absolute_import
 
 import struct
-import dpkt
-from decorators import deprecated
+import codecs
+
+from . import dpkt
+from . import ethernet
+from .decorators import deprecated
+from .compat import compat_izip
 
 GRE_CP = 0x8000  # Checksum Present
 GRE_RP = 0x4000  # Routing Present
@@ -20,11 +25,19 @@ GRE_opt_fields = (
 
 
 class GRE(dpkt.Packet):
+    """Generic Routing Encapsulation.
+
+    TODO: Longer class information....
+
+    Attributes:
+        __hdr__: Header fields of GRE.
+        TODO.
+    """
+
     __hdr__ = (
         ('flags', 'H', 0),
         ('p', 'H', 0x0800),  # ETH_TYPE_IP
     )
-    _protosw = {}
     sre = ()
 
     @property
@@ -43,21 +56,6 @@ class GRE(dpkt.Packet):
     def recur(self, v):
         self.flags = (self.flags & ~0xe0) | ((v & 0x7) << 5)
 
-    # Deprecated methods, will be removed in the future
-    # =================================================
-    @deprecated('v')
-    def get_v(self): return self.v
-
-    @deprecated('v')
-    def set_v(self, v): self.v = v
-
-    @deprecated('recur')
-    def get_recur(self): return self.recur
-
-    @deprecated('recur')
-    def set_recur(self, v): self.recur = v
-    # =================================================
-
     class SRE(dpkt.Packet):
         __hdr__ = [
             ('family', 'H', 0),
@@ -88,9 +86,9 @@ class GRE(dpkt.Packet):
         if fields:
             fmt = ''.join(fmts)
             fmtlen = struct.calcsize(fmt)
-            vals = struct.unpack(fmt, self.data[:fmtlen])
+            vals = struct.unpack("!" + fmt, self.data[:fmtlen])
             self.data = self.data[fmtlen:]
-            self.__dict__.update(dict(zip(fields, vals)))
+            self.__dict__.update(dict(compat_izip(fields, vals)))
         if self.flags & GRE_RP:
             l = []
             while True:
@@ -100,25 +98,52 @@ class GRE(dpkt.Packet):
                 if not sre.len:
                     break
             self.sre = l
-        self.data = ethernet.Ethernet._typesw[self.p](self.data)
-        setattr(self, self.data.__class__.__name__.lower(), self.data)
+        try:
+            self.data = ethernet.Ethernet._typesw[self.p](self.data)
+            setattr(self, self.data.__class__.__name__.lower(), self.data)
+        except (KeyError, dpkt.UnpackError):
+            # data alrady set
+            pass
 
     def __len__(self):
-        opt_fmtlen = struct.calcsize(''.join(self.opt_fields_fmts()[1]))
+        opt_fmtlen = struct.calcsize(b''.join(self.opt_fields_fmts()[1]))
         return self.__hdr_len__ + opt_fmtlen + sum(map(len, self.sre)) + len(self.data)
 
-    def __str__(self):
+    def __bytes__(self):
         fields, fmts = self.opt_fields_fmts()
         if fields:
             vals = []
             for f in fields:
                 vals.append(getattr(self, f))
-            opt_s = struct.pack(''.join(fmts), *vals)
+            opt_s = struct.pack(b''.join(fmts), *vals)
         else:
-            opt_s = ''
-        return self.pack_hdr() + opt_s + ''.join(map(str, self.sre)) + str(self.data)
+            opt_s = b''
+        return self.pack_hdr() + opt_s + b''.join(map(bytes, self.sre)) + bytes(self.data)
 
-# XXX - auto-load GRE dispatch table from Ethernet dispatch table
-import ethernet
 
-GRE._protosw.update(ethernet.Ethernet._typesw)
+def test_gre_v1():
+    # Runs all the test associated with this class/file
+    s = codecs.decode("3081880a0067178000068fb100083a76", 'hex') + b"A" * 103
+    g = GRE(s)
+
+    assert g.v == 1
+    assert g.p == 0x880a
+    assert g.seq == 430001
+    assert g.ack == 539254
+    assert g.callid == 6016
+    assert g.len == 103
+    assert g.data == b"A" * 103
+
+    s = codecs.decode("3001880a00b2001100083ab8", 'hex') + b"A" * 178
+    g = GRE(s)
+
+    assert g.v == 1
+    assert g.p == 0x880a
+    assert g.seq == 539320
+    assert g.callid == 17
+    assert g.len == 178
+    assert g.data == b"A" * 178
+
+
+if __name__ == '__main__':
+    test_gre_v1()
