@@ -1,8 +1,9 @@
 # $Id: rx.py 23 2006-11-08 15:45:33Z jonojono $
 # -*- coding: utf-8 -*-
 """Rx Protocol."""
+from __future__ import absolute_import
 
-import dpkt
+from . import dpkt
 
 # Types
 DATA = 0x01
@@ -30,6 +31,15 @@ SEC_RXKAD_ENC = 0x03
 
 
 class Rx(dpkt.Packet):
+    """Rx Protocol.
+
+    TODO: Longer class information....
+
+    Attributes:
+        __hdr__: Header fields of Rx.
+        TODO.
+    """
+    
     __hdr__ = (
         ('epoch', 'I', 0),
         ('cid', 'I', 0),
